@@ -33,15 +33,21 @@
 #include <akonadi/itemfetchjob.h>
 #include <akonadi/itemfetchscope.h>
 #include <akonadi/mimetypechecker.h>
+#include <Akonadi/ItemSearchJob>
 
 #include <nepomuk/resource.h>
 #include <nepomuk/tag.h>
+#include <nepomuk/andterm.h>
+#include <nepomuk/comparisonterm.h>
+#include <nepomuk/literalterm.h>
+#include <nepomuk/resourcetypeterm.h>
 
 #include <KLocale>
 #include <KUrl>
 #include <KProcess>
 #include <KMessageBox>
 #include <KStandardDirs>
+#include <KIdleTime>
 
 #include <Soprano/Vocabulary/NAO>
 
@@ -58,6 +64,7 @@
 #include <QtDBus/QDBusReply>
 #include <QtDBus/QDBusConnectionInterface>
 #include <QtDBus/QDBusInterface>
+#include <KConfigGroup>
 
 using namespace Akonadi;
 
@@ -73,23 +80,33 @@ NepomukFeederAgentBase::NepomukFeederAgentBase(const QString& id) :
   mPendingJobs( 0 ),
   mNrlModel( 0 ),
   mStrigiIndexManager( 0 ),
+  mIndexCompatLevel( 1 ),
   mNepomukStartupAttempted( false ),
   mInitialUpdateDone( false ),
-  mNeedsStrigi( false )
+  mNeedsStrigi( false ),
+  mSelfTestPassed( false ),
+  mSystemIsIdle( false ),
+  mIdleDetectionDisabled( false )
 {
   // initialize Nepomuk
   Nepomuk::ResourceManager::instance()->init();
   mNrlModel = new Soprano::NRLModel( Nepomuk::ResourceManager::instance()->mainModel() );
 
-  changeRecorder()->setChangeRecordingEnabled( false );
   changeRecorder()->fetchCollection( true );
+  changeRecorder()->itemFetchScope().setAncestorRetrieval( ItemFetchScope::Parent );
 
-  mNepomukStartupTimeout.setInterval( 60 * 1000 );
+  mNepomukStartupTimeout.setInterval( 300 * 1000 );
   mNepomukStartupTimeout.setSingleShot( true );
   connect( &mNepomukStartupTimeout, SIGNAL(timeout()), SLOT(selfTest()) );
-  connect( QDBusConnection::sessionBus().interface(), SIGNAL(serviceOwnerChanged(QString,QString,QString)), SLOT(serviceOwnerChanged(QString,QString,QString)) );
+  connect( Nepomuk::ResourceManager::instance(), SIGNAL(nepomukSystemStarted()), SLOT(selfTest()) );
+  connect( Nepomuk::ResourceManager::instance(), SIGNAL(nepomukSystemStopped()), SLOT(selfTest()) );
+  connect( this, SIGNAL(fullyIndexed()), this, SLOT(slotFullyIndexed()) );
 
-  setOnline( false );
+  connect( KIdleTime::instance(), SIGNAL(timeoutReached(int)), SLOT(systemIdle()) );
+  connect( KIdleTime::instance(), SIGNAL(resumingFromIdle()), SLOT(systemResumed()) );
+  KIdleTime::instance()->addIdleTimeout( 10 * 1000 );
+
+  checkOnline();
   QTimer::singleShot( 0, this, SLOT(selfTest()) );
 }
 
@@ -104,18 +121,44 @@ void NepomukFeederAgentBase::itemAdded(const Akonadi::Item& item, const Akonadi:
 {
   if ( entityIsHidden( collection ) )
     return;
-  if ( item.hasPayload() )
-    updateItem( item, createGraphForEntity( item ) );
+
+  if ( item.hasPayload() ) {
+    const QUrl graph = createGraphForEntity( item );
+    mNrlModel->addStatement( item.url(), Akonadi::ItemSearchJob::akonadiItemIdUri(),
+                             QUrl( QString::number( item.id() ) ), graph );
+    updateItem( item, graph );
+  } else {
+    const ItemFetchScope scope = fetchScopeForCollection( collection );
+    if ( scope.fullPayload() || !scope.payloadParts().isEmpty() ) {
+      ItemFetchJob *job = new ItemFetchJob( item );
+      job->setFetchScope( scope );
+      connect( job, SIGNAL( itemsReceived( Akonadi::Item::List ) ),
+               SLOT( notificationItemsReceived( Akonadi::Item::List ) ) );
+    }
+  }
 }
 
-void NepomukFeederAgentBase::itemChanged(const Akonadi::Item& item, const QSet< QByteArray >&)
+void NepomukFeederAgentBase::itemChanged(const Akonadi::Item& item, const QSet< QByteArray >& partIdentifiers)
 {
   if ( entityIsHidden( item.parentCollection() ) )
     return;
   // TODO: check part identfiers if anything interesting changed at all
   if ( item.hasPayload() ) {
     removeEntityFromNepomuk( item );
-    updateItem( item, createGraphForEntity( item ) );
+
+    const QUrl graph = createGraphForEntity( item );
+    mNrlModel->addStatement( item.url(), Akonadi::ItemSearchJob::akonadiItemIdUri(),
+                             QUrl( QString::number( item.id() ) ), graph );
+    updateItem( item, graph );
+  } else {
+    const Collection collection = item.parentCollection();
+    const ItemFetchScope scope = fetchScopeForCollection( collection );
+    if ( scope.fullPayload() || !scope.payloadParts().isEmpty() ) {
+      ItemFetchJob *job = new ItemFetchJob( item );
+      job->setFetchScope( scope );
+      connect( job, SIGNAL( itemsReceived( Akonadi::Item::List ) ),
+               SLOT( notificationItemsReceived( Akonadi::Item::List ) ) );
+    }
   }
 }
 
@@ -168,13 +211,20 @@ void NepomukFeederAgentBase::collectionsReceived(const Akonadi::Collection::List
 
     mCollectionQueue.append( collection );
   }
-  processNextCollection();
+
+  if ( mPendingJobs == 0 ) {
+    processNextCollection();
+  }
 }
 
 void NepomukFeederAgentBase::processNextCollection()
 {
-  if ( mCurrentCollection.isValid() || mCollectionQueue.isEmpty() )
+  if ( mCurrentCollection.isValid() )
     return;
+  if ( mCollectionQueue.isEmpty() ) {
+    emit fullyIndexed();
+    return;
+  }
   mCurrentCollection = mCollectionQueue.takeFirst();
   emit status( AgentBase::Running, i18n( "Indexing collection '%1'...", mCurrentCollection.name() ) );
   kDebug() << "Indexing collection" << mCurrentCollection.name();
@@ -200,11 +250,20 @@ void NepomukFeederAgentBase::itemHeadersReceived(const Akonadi::Item::List& item
     // update item if it does not exist
     if ( !Nepomuk::ResourceManager::instance()->mainModel()->containsAnyStatement( item.url(), Soprano::Node(), Soprano::Node() ) )
       itemsToUpdate.append( item );
+
+    // the item exists. Check if it has an item ID property, otherwise re-index it.
+    else {
+      if ( !Nepomuk::ResourceManager::instance()->mainModel()->containsAnyStatement( item.url(),
+                                   Akonadi::ItemSearchJob::akonadiItemIdUri(), Soprano::Node() ) ) {
+        removeEntityFromNepomuk( item );
+        itemsToUpdate.append( item );
+      }
+    }
   }
 
   if ( !itemsToUpdate.isEmpty() ) {
     ItemFetchJob *itemFetch = new ItemFetchJob( itemsToUpdate, this );
-    itemFetch->setFetchScope( changeRecorder()->itemFetchScope() );
+    itemFetch->setFetchScope( fetchScopeForCollection( mCurrentCollection ) );
     connect( itemFetch, SIGNAL(itemsReceived(Akonadi::Item::List)), SLOT(itemsReceived(Akonadi::Item::List)) );
     connect( itemFetch, SIGNAL(result(KJob*)), SLOT(itemFetchResult(KJob*)) );
     ++mPendingJobs;
@@ -232,12 +291,30 @@ void NepomukFeederAgentBase::itemsReceived(const Akonadi::Item::List& items)
   foreach ( Item item, items ) {
     // we only get here if the item is not anywhere in Nepomuk yet, so no need to delete it
     item.setParentCollection( mCurrentCollection );
-    updateItem( item, createGraphForEntity( item ) );
+    const QUrl graph = createGraphForEntity( item );
+    mNrlModel->addStatement( item.url(), Akonadi::ItemSearchJob::akonadiItemIdUri(),
+                             QUrl( QString::number( item.id() ) ), graph );
+    updateItem( item, graph );
   }
   mProcessedAmount += items.count();
   emit percent( (mProcessedAmount * 100) / (mTotalAmount * 100) );
 }
 
+void NepomukFeederAgentBase::notificationItemsReceived(const Akonadi::Item::List& items)
+{
+  kDebug() << items.size();
+  foreach ( const Item &item, items ) {
+    if ( !item.hasPayload() ) {
+      continue;
+    }
+    removeEntityFromNepomuk( item );
+
+    const QUrl graph = createGraphForEntity( item );
+    mNrlModel->addStatement( item.url(), Akonadi::ItemSearchJob::akonadiItemIdUri(),
+                             QUrl( QString::number( item.id() ) ), graph );
+    updateItem( item, graph );
+  }
+}
 
 void NepomukFeederAgentBase::tagsFromCategories(NepomukFast::Resource& resource, const QStringList& categories)
 {
@@ -253,9 +330,10 @@ void NepomukFeederAgentBase::tagsFromCategories(NepomukFast::Resource& resource,
 void NepomukFeederAgentBase::selfTest()
 {
   QStringList errorMessages;
+  mSelfTestPassed = false;
 
   // if Nepomuk is not running, try to start it
-  if ( !mNepomukStartupAttempted && !QDBusConnection::sessionBus().interface()->isServiceRegistered( QLatin1String( "org.kde.NepomukStorage" ) ) ) {
+  if ( !mNepomukStartupAttempted && !Nepomuk::ResourceManager::instance()->initialized() ) {
     KProcess process;
     const QString nepomukserver = KStandardDirs::findExe( QLatin1String("nepomukserver") );
     if ( process.startDetached( nepomukserver ) == 0 ) {
@@ -264,14 +342,14 @@ void NepomukFeederAgentBase::selfTest()
       mNepomukStartupAttempted = true;
       mNepomukStartupTimeout.start();
       // wait for Nepomuk to start
-      setOnline( false );
+      checkOnline();
       emit status( Broken, i18n( "Waiting for the Nepomuk server to start..." ) );
       return;
     }
   }
 
   // if it is already running, check if the backend is correct
-  if ( QDBusConnection::sessionBus().interface()->isServiceRegistered( QLatin1String( "org.kde.NepomukStorage" ) ) ) {
+  if ( Nepomuk::ResourceManager::instance()->initialized() ) {
     static const QStringList backendBlacklist = QStringList() << QLatin1String( "redland" );
     // check which backend is used
     QDBusInterface interface( QLatin1String( "org.kde.NepomukStorage" ), QLatin1String( "/nepomukstorage" ) );
@@ -283,6 +361,11 @@ void NepomukFeederAgentBase::selfTest()
     } else {
       errorMessages.append( i18n( "Calling the Nepomuk storage service failed: '%1'.", reply.error().message() ) );
     }
+  } else if ( mNepomukStartupAttempted && mNepomukStartupTimeout.isActive() ) {
+    // still waiting for Nepomuk to start
+    setOnline( false );
+    emit status( Broken, i18n( "Waiting for the Nepomuk server to start..." ) );
+    return;
   } else {
     errorMessages.append( i18n( "Nepomuk is not running." ) );
   }
@@ -291,13 +374,15 @@ void NepomukFeederAgentBase::selfTest()
   if ( !mStrigiIndexManager && mNeedsStrigi ) {
     mStrigiIndexManager = Strigi::IndexPluginLoader::createIndexManager( "nepomukbackend", 0 );
     if ( !mStrigiIndexManager )
-      errorMessages.append( i18n( "Soprano backend for Strigi is not available." ) );
+      errorMessages.append( i18n( "Nepomuk backend for Strigi is not available." ) );
   }
 
   if ( errorMessages.isEmpty() ) {
-    setOnline( true );
+    mSelfTestPassed = true;
     mNepomukStartupAttempted = false; // everything worked, we can try again if the server goes down later
-    if ( !mInitialUpdateDone ) {
+    mNepomukStartupTimeout.stop();
+    checkOnline();
+    if ( !mInitialUpdateDone && needsReIndexing() ) {
       mInitialUpdateDone = true;
       QTimer::singleShot( 0, this, SLOT(updateAll()) );
     } else {
@@ -306,7 +391,7 @@ void NepomukFeederAgentBase::selfTest()
     return;
   }
 
-  setOnline( false );
+  checkOnline();
 
   QString message = i18n( "<b>Nepomuk Indexing Agents Have Been Disabled</b><br/>"
                           "The Nepomuk service is not available or fully operational and attempts to rectify this have failed. "
@@ -320,19 +405,10 @@ void NepomukFeederAgentBase::selfTest()
   emit status( Broken, i18n( "Nepomuk not operational" ) );
   if ( !QDBusConnection::sessionBus().registerService( QLatin1String( "org.kde.pim.nepomukfeeder.selftestreport" ) ) )
     return;
-  //KMessageBox::error( 0, message, i18n( "Nepomuk Indexing Disabled" ), KMessageBox::Notify | KMessageBox::AllowLink );
+  KMessageBox::error( 0, message, i18n( "Nepomuk Indexing Disabled" ), KMessageBox::Notify | KMessageBox::AllowLink );
   QDBusConnection::sessionBus().unregisterService( QLatin1String( "org.kde.pim.nepomukfeeder.selftestreport" ) );
 }
 
-void NepomukFeederAgentBase::serviceOwnerChanged(const QString& name, const QString& oldOwner, const QString& newOwner)
-{
-  Q_UNUSED( oldOwner );
-  Q_UNUSED( newOwner );
-
-  if ( name == QLatin1String("org.kde.NepomukStorage") )
-    selfTest();
-}
-
 NepomukFast::PersonContact NepomukFeederAgentBase::findOrCreateContact(const QString& emailAddress, const QString& name, const QUrl& graphUri, bool* found)
 {
   //
@@ -340,17 +416,24 @@ NepomukFast::PersonContact NepomukFeederAgentBase::findOrCreateContact(const QSt
   // are case insensitive. But for the moment we stick to it and hope Nepomuk
   // alignment fixes any duplicates
   //
-  SelectSparqlBuilder::BasicGraphPattern graph;
+  Nepomuk::Query::Query query;
+  Nepomuk::Query::AndTerm andTerm;
+  Nepomuk::Query::ResourceTypeTerm personTypeTerm( Vocabulary::NCO::PersonContact() );
+  andTerm.addSubTerm( personTypeTerm );
   if ( emailAddress.isEmpty() ) {
-    graph.addTriple( QLatin1String( "?person" ), Vocabulary::NCO::fullname(), name );
+    const Nepomuk::Query::ComparisonTerm nameTerm( Vocabulary::NCO::fullname(), Nepomuk::Query::LiteralTerm( name ),
+                                                   Nepomuk::Query::ComparisonTerm::Equal );
+    andTerm.addSubTerm( nameTerm );
   } else {
-    graph.addTriple( QLatin1String( "?person" ), Vocabulary::NCO::hasEmailAddress(), SparqlBuilder::QueryVariable( "?email" ) );
-    graph.addTriple( QLatin1String( "?email" ), Vocabulary::NCO::emailAddress(), emailAddress );
+    const Nepomuk::Query::ComparisonTerm addrTerm( Vocabulary::NCO::emailAddress(), Nepomuk::Query::LiteralTerm( emailAddress ),
+                                                   Nepomuk::Query::ComparisonTerm::Equal );
+    const Nepomuk::Query::ComparisonTerm mailTerm( Vocabulary::NCO::hasEmailAddress(), addrTerm );
+    andTerm.addSubTerm( mailTerm );
   }
-  SelectSparqlBuilder qb;
-  qb.setGraphPattern( graph );
-  qb.addQueryVariable( QLatin1String( "?person" ) );
-  Soprano::QueryResultIterator it = Nepomuk::ResourceManager::instance()->mainModel()->executeQuery( qb.query(), Soprano::Query::QueryLanguageSparql );
+  query.setTerm( andTerm );
+  query.setLimit( 1 );
+  
+  Soprano::QueryResultIterator it = Nepomuk::ResourceManager::instance()->mainModel()->executeQuery( query.toSparqlQuery(), Soprano::Query::QueryLanguageSparql );
 
   if ( it.next() ) {
     if ( found ) *found = true;
@@ -395,4 +478,67 @@ void NepomukFeederAgentBase::indexData(const KUrl& url, const QByteArray& data,
   idx.index( &sr );
 }
 
+ItemFetchScope NepomukFeederAgentBase::fetchScopeForCollection(const Akonadi::Collection& collection)
+{
+  return changeRecorder()->itemFetchScope();
+}
+
+void NepomukFeederAgentBase::setIndexCompatibilityLevel(int level)
+{
+  mIndexCompatLevel = level;
+}
+
+void NepomukFeederAgentBase::disableIdleDetection( bool value )
+{
+  mIdleDetectionDisabled = value;
+}
+
+bool NepomukFeederAgentBase::needsReIndexing() const
+{
+  const KConfigGroup grp( componentData().config(), "InitialIndexing" );
+  return mIndexCompatLevel > grp.readEntry( "IndexCompatLevel", 0 );
+}
+
+void NepomukFeederAgentBase::slotFullyIndexed()
+{
+  KConfigGroup grp( componentData().config(), "InitialIndexing" );
+  grp.writeEntry( "IndexCompatLevel", mIndexCompatLevel );
+  grp.sync();
+}
+
+void NepomukFeederAgentBase::doSetOnline(bool online)
+{
+  changeRecorder()->setChangeRecordingEnabled( !online );
+  Akonadi::AgentBase::doSetOnline( online );
+}
+
+void NepomukFeederAgentBase::checkOnline()
+{
+  if ( mIdleDetectionDisabled )
+    setOnline( mSelfTestPassed );
+  else
+    setOnline( mSelfTestPassed && mSystemIsIdle );
+}
+
+void NepomukFeederAgentBase::systemIdle()
+{
+  if ( mIdleDetectionDisabled )
+    return;
+
+  emit status( Idle, i18n( "System idle, ready to index data." ) );
+  mSystemIsIdle = true;
+  KIdleTime::instance()->catchNextResumeEvent();
+  checkOnline();
+}
+
+void NepomukFeederAgentBase::systemResumed()
+{
+  if ( mIdleDetectionDisabled )
+    return;
+
+  emit status( Idle, i18n( "System busy, indexing suspended." ) );
+  mSystemIsIdle = false;
+  checkOnline();
+}
+
 #include "nepomukfeederagentbase.moc"
