@@ -22,7 +22,6 @@
 
 #include "kmigratorbase.h"
 
-#include "kmindexreader/messagestatus.h"
 #include "libmaildir/maildir.h"
 #include "mixedmaildirstore.h"
 #include "subscriptionjob_p.h"
@@ -41,15 +40,13 @@
 #include <akonadi/itemfetchscope.h>
 #include <akonadi/monitor.h>
 #include <akonadi/session.h>
+#include <akonadi/kmime/messagestatus.h>
 
 #include <Nepomuk/Tag>
 
 #include <KConfigGroup>
-#include <KGlobal>
 #include <KLocale>
-#include <KStandardDirs>
 
-#include <QFileInfo>
 #include <QSet>
 #include <QVariant>
 
@@ -230,7 +227,7 @@ void ImapCacheCollectionMigrator::Private::processNextItem()
 
   // don't import items that are marked deleted. These come from normal/online IMAP caches
   // which didn't get their mbox cache files compacted
-  KPIM::MessageStatus status;
+  Akonadi::MessageStatus status;
   status.setStatusFromFlags( item.flags() );
   if ( status.isDeleted() ) {
     //kDebug() << "Cache item" << item.remoteId() << "is marked as Deleted. Skip it";
