@@ -24,9 +24,12 @@
 
 #include "messagetag.h"
 
+#ifndef KDEPIM_NO_NEPOMUK
 // avoid metatype.h from interfering
 #include <Nepomuk/Variant>
 #define POP3_METATYPE_H
+#include <Nepomuk/Tag>
+#endif
 
 #include "imapsettings.h"
 #include "pop3settings.h"
@@ -41,7 +44,6 @@
 
 #include "collectionannotationsattribute.h"
 
-#include <KIMAP/LoginJob>
 #include <Mailtransport/Transport>
 
 #include <akonadi/agentmanager.h>
@@ -52,19 +54,17 @@ using Akonadi::AgentManager;
 using Akonadi::AgentInstance;
 using Akonadi::AgentInstanceCreateJob;
 
-#include <Nepomuk/Tag>
-
 #include <KConfig>
 #include <KConfigGroup>
 #include <KDebug>
 #include <KStandardDirs>
-#include <KLocalizedString>
 #include <KMessageBox>
 #include <KSharedConfig>
 #include <KCursor>
 #include <kwallet.h>
 #include <QApplication>
 #include <kstringhandler.h>
+#include <KLocalizedString>
 using KWallet::Wallet;
 
 using namespace KMail;
@@ -216,6 +216,7 @@ void KMailMigrator::migrateTags()
 
     const QString shortcut = group.readEntry( QLatin1String( "shortcut" ), QString() );
 
+#ifndef KDEPIM_NO_NEPOMUK
     Nepomuk::Tag nepomukTag( label );
     nepomukTag.setLabel( name );
     nepomukTag.setSymbols( QStringList( iconName ) );
@@ -244,6 +245,7 @@ void KMailMigrator::migrateTags()
              << "shortcut=" << shortcut;
 
     newlyMigratedTags << groupName;
+#endif
   }
 
   if ( !newlyMigratedTags.isEmpty() ) {
@@ -356,6 +358,7 @@ void KMailMigrator::migrationDone()
     }
   }
   cleanupConfigFile();
+  migrateConfigurationDialogRestriction();
   deleteLater();
 }
 
@@ -385,6 +388,18 @@ OrgKdeAkonadiPOP3SettingsInterface* KMailMigrator::createPop3SettingsInterface(
   return iface;
 }
 
+void KMailMigrator::migrateConfigurationDialogRestriction()
+{
+  if ( mConfig->hasGroup( "ConfigurationDialogRestrictions" ) ) {
+    KSharedConfigPtr resourcebaseconfig = KSharedConfig::openConfig( QLatin1String( "resourcebaserc" ) );
+    KConfigGroup oldGroup = mConfig->group( "ConfigurationDialogRestrictions" );
+    KConfigGroup newGroup = resourcebaseconfig->group( "ConfigurationDialogRestrictions" );
+    oldGroup.copyTo( &newGroup );
+    oldGroup.deleteGroup();
+  }
+
+}
+
 void KMailMigrator::cleanupConfigFile()
 {
   mIt = mAccounts.begin();
@@ -457,6 +472,28 @@ void KMailMigrator::migrationFailed( const QString &errorMsg,
 void KMailMigrator::migrationCompleted( const AgentInstance &instance, bool doMigrateNext )
 {
   const KConfigGroup group( mConfig, mCurrentAccount );
+  const QString accountId = group.readEntry( QLatin1String( "Id" ) );
+
+  // check if the account is used in filters
+  const QStringList filterGroups = mConfig->groupList().filter( QRegExp( "Filter #\\d+" ) );
+  //kDebug( KDE_DEFAULT_DEBUG_AREA ) << "filterGroups=" << filterGroups;
+  Q_FOREACH( const QString &groupName, filterGroups ) {
+    KConfigGroup filterGroup( mConfig, groupName );
+
+    QStringList accountsSet = filterGroup.readEntry( QLatin1String( "accounts-set" ), QStringList() );
+    //if ( !accountsSet.isEmpty() ) {
+    //  kDebug( KDE_DEFAULT_DEBUG_AREA ) << "accountsSet=" << accountsSet;
+    //}
+    const int index = accountsSet.indexOf( accountId );
+    if ( index != -1 ) {
+      kDebug( KDE_DEFAULT_DEBUG_AREA ) << "replacing account id" << accountId
+                                       << "in filter" << groupName
+                                       << "with resource id" << instance.identifier();
+      accountsSet.replace( index, instance.identifier() );
+      filterGroup.writeEntry( QLatin1String( "accounts-set" ), accountsSet );
+    }
+  }
+
   setMigrationState( group.readEntry( "Id" ), Complete, instance.identifier(),
                      group.readEntry( "Type" ).toLower() );
   emit message( Success, i18n( "Migration of '%1' succeeded.", group.readEntry( "Name" ) ) );
@@ -504,9 +541,9 @@ bool KMailMigrator::evaluateCacheHandlingOptions()
 
     const QString message =
       i18nc( "@info", "<para>Cached IMAP accounts have a local copy of the server's data.</para>"
-                      "<para>These copies can be kept in local folders or be deleted"
+                      "<para>These copies can either be kept in local folders, or be deleted "
                       "after import.</para>"
-                      "<note>Mail that did not get imported will always be kept in local folders"
+                      "<note>Mail that was not imported will always be kept in local folders."
                       "</note>" );
 
     int result = KMessageBox::questionYesNoCancel( 0, message, i18n( "KMail 2 Migration" ),
@@ -673,7 +710,9 @@ void KMailMigrator::migrateImapAccount( KJob *job, bool disconnected )
   iface->setSieveAlternateUrl( config.readEntry( "sieve-alternate-url" ) );
   iface->setSieveVacationFilename( config.readEntry( "sieve-vacation-filename", "kmail-vacation.siv" ) );
   iface->setDisconnectedModeEnabled( disconnected );
-  iface->setAutomaticExpungeEnabled( config.readEntry("auto-expunge", true ) );
+  if ( !disconnected ) {
+    iface->setAutomaticExpungeEnabled( config.readEntry("auto-expunge", true ) );
+  }
   const bool useDefaultIdentity = config.readEntry( "use-default-identity", true );
   iface->setUseDefaultIdentity( useDefaultIdentity );
   if ( !useDefaultIdentity )
@@ -1187,8 +1226,8 @@ void KMailMigrator::imapFoldersMigrationFinished( const AgentInstance &instance,
 
       emit message( Info,
                     i18ncp( "@info:status",
-                            "Scheduling background task for recovering unimported messages.",
-                            "Scheduling background task for recovering unimported messages. "
+                            "Scheduling background task for recovering non-imported messages.",
+                            "Scheduling background task for recovering non-imported messages. "
                             "Now at a total of %1 such tasks",
                              mRunningCacheImporterCount ) );
 
