@@ -20,6 +20,7 @@
 #include "kolabproxyresource.h"
 
 #include "settings.h"
+#include "setupkolab.h"
 #include "settingsadaptor.h"
 #include "collectionannotationsattribute.h"
 #include "addressbookhandler.h"
@@ -50,6 +51,15 @@
 #include <QtDBus/QDBusConnection>
 #include <QSet>
 
+#ifdef RUNTIME_PLUGINS_STATIC
+#include <QtPlugin>
+
+Q_IMPORT_PLUGIN(akonadi_serializer_mail)
+Q_IMPORT_PLUGIN(akonadi_serializer_addressee)
+Q_IMPORT_PLUGIN(akonadi_serializer_kcalcore)
+Q_IMPORT_PLUGIN(akonadi_serializer_contactgroup)
+#endif
+
 using namespace Akonadi;
 
 static const char KOLAB_COLLECTION[] = "KolabCollection";
@@ -98,20 +108,6 @@ KolabProxyResource::KolabProxyResource( const QString &id )
   connect(m_collectionMonitor, SIGNAL(collectionChanged(const Akonadi::Collection &)), this, SLOT(imapCollectionChanged(const Akonadi::Collection &)));
   connect(m_collectionMonitor, SIGNAL(collectionMoved(Akonadi::Collection,Akonadi::Collection,Akonadi::Collection)), this, SLOT(imapCollectionMoved(Akonadi::Collection,Akonadi::Collection,Akonadi::Collection)) );
 
-  m_root.setName( identifier() );
-  m_root.setParentCollection( Collection::root() );
-  EntityDisplayAttribute *attr = m_root.attribute<EntityDisplayAttribute>( Collection::AddIfMissing );
-  attr->setDisplayName( i18n("Kolab") );
-  attr->setIconName( "kolab" );
-  m_root.setContentMimeTypes( QStringList() << Collection::mimeType() );
-  m_root.setRemoteId( identifier() );
-  m_root.setRights( Collection::ReadOnly );
-  CachePolicy policy;
-  policy.setInheritFromParent( false );
-  policy.setCacheTimeout( -1 );
-  policy.setLocalParts( QStringList() << QLatin1String( "ALL" ) );
-  m_root.setCachePolicy( policy );
-
   setName( i18n("Kolab") );
 
   // among other things, this ensures that m_root actually exists when a new imap folder is added
@@ -138,8 +134,6 @@ void KolabProxyResource::retrieveCollectionsTreeDone(KJob* job)
     Collection::List imapCollections = qobject_cast<CollectionTreeBuilder*>( job )->allCollections();
 
     Collection::List kolabCollections;
-    kolabCollections.append( m_root );
-
     Q_FOREACH(const Collection &collection, imapCollections)
       kolabCollections.append( createCollection(collection) );
     collectionsRetrieved( kolabCollections );
@@ -153,11 +147,14 @@ void KolabProxyResource::retrieveItems( const Collection &collection )
   m_retrieveState = RetrieveItems;
   ItemFetchJob *job = new ItemFetchJob( kolabToImap( collection ) );
   job->fetchScope().fetchFullPayload();
+  job->setProperty( "resultCanBeEmpty", true );
+
   connect(job, SIGNAL(result(KJob*)), this, SLOT(retrieveItemFetchDone(KJob *)));
 }
 
 bool KolabProxyResource::retrieveItem( const Item &item, const QSet<QByteArray> &parts )
 {
+  Q_UNUSED( parts );
   kDebug() << "RETRIEVEITEM";
   m_retrieveState = RetrieveItem;
   ItemFetchJob *job = new ItemFetchJob( kolabToImap( item ) );
@@ -172,11 +169,17 @@ void KolabProxyResource::retrieveItemFetchDone(KJob *job)
     kWarning( ) << "Error on item fetch:" << job->errorText();
     cancelTask();
   } else {
+    const bool resultCanBeEmpty = job->property( "resultCanBeEmpty" ).isValid();
+
     Item::Id collectionId = -1;
-    Item::List items = qobject_cast<ItemFetchJob*>(job)->items();
-    if (items.size() < 1) {
-      kWarning() << "Items is emtpy";
-      cancelTask();
+    const Item::List items = qobject_cast<ItemFetchJob*>(job)->items();
+    if ( items.size() < 1 ) {
+      if ( resultCanBeEmpty ) {
+        itemsRetrieved( Akonadi::Item::List() );
+      } else {
+        kWarning() << "Items is emtpy";
+        cancelTask();
+      }
       return;
     }
     collectionId = items[0].storageCollectionId();
@@ -202,6 +205,7 @@ void KolabProxyResource::retrieveItemFetchDone(KJob *job)
 void KolabProxyResource::aboutToQuit()
 {
   qDeleteAll(m_monitoredCollections);
+  m_monitoredCollections.clear();
 }
 
 void KolabProxyResource::configure( WId windowId )
@@ -215,7 +219,15 @@ void KolabProxyResource::configure( WId windowId )
   // "on top of parent" behavior if the running window manager applies any kind
   // of focus stealing prevention technique
 
-  emit configurationDialogAccepted();
+  QPointer<SetupKolab> kolabConfigDialog( new SetupKolab( this, windowId ) );
+  if ( kolabConfigDialog->exec() == QDialog::Accepted ) {
+    emit configurationDialogAccepted();
+  }
+  else {
+    emit configurationDialogRejected();
+  }
+
+  delete kolabConfigDialog;
 }
 
 void KolabProxyResource::itemAdded( const Item &item, const Collection &collection )
@@ -363,7 +375,14 @@ void KolabProxyResource::itemRemoved( const Item &item )
 
 void KolabProxyResource::collectionAdded(const Akonadi::Collection& collection, const Akonadi::Collection& parent)
 {
-  Q_ASSERT( !KolabHandler::kolabTypeForCollection( collection ).isEmpty() );
+  if ( KolabHandler::kolabTypeForCollection( collection ).isEmpty() ) {
+    kWarning() << "Collection " << collection.name() << collection.id() << collection.isValid()
+               << "doesn't have kolab type set. isValid = "
+               << "; parent is " << parent.name() << parent.id() << parent.isValid();
+    cancelTask( QLatin1String( "Collection doesn't have kolab type." ) );
+    Q_ASSERT_X( false, "collectionAdded", "Colection doesn't have kolab type set. Crashing..." );
+    return;
+  }
 
   Collection imapCollection( collection );
   imapCollection.setId( -1 );
@@ -448,6 +467,7 @@ void KolabProxyResource::collectionChanged(const Akonadi::Collection& collection
   applyAttributesToImap( imapCollection, collection );
 
   CollectionModifyJob *job = new CollectionModifyJob( imapCollection, this );
+  Q_UNUSED( job );
   // TODO wait for the result
   changeCommitted( collection );
 }
@@ -464,6 +484,7 @@ void KolabProxyResource::collectionRemoved(const Akonadi::Collection& collection
   Collection imapCollection = kolabToImap( collection );
 
   CollectionDeleteJob *job = new CollectionDeleteJob( imapCollection, this );
+  Q_UNUSED( job );
   // TODO wait for result
   changeCommitted( collection );
 }
@@ -543,6 +564,7 @@ void KolabProxyResource::imapItemRemoved(const Item& item)
     handler->itemDeleted(item);
   }
   ItemDeleteJob *job = new ItemDeleteJob( kolabItem, this );
+  Q_UNUSED( job );
 }
 
 void KolabProxyResource::imapItemMoved(const Akonadi::Item& item, const Akonadi::Collection& collectionSource, const Akonadi::Collection& collectionDestination)
@@ -578,7 +600,7 @@ void KolabProxyResource::imapCollectionChanged(const Collection &collection)
   if ( collection.resource() == identifier() ) // just to be sure...
     return;
 
-  kDebug() << "IMAPCOLLECTIONCHANGED";
+  //kDebug() << "IMAPCOLLECTIONCHANGED";
   if ( !m_monitoredCollections.contains(collection.id()) ) {
     // check if this is a Kolab folder at all, if yet something is wrong
     CollectionAnnotationsAttribute *annotationsAttribute =
@@ -598,6 +620,7 @@ void KolabProxyResource::imapCollectionChanged(const Collection &collection)
     // if that fails it's not in our tree -> we don't care
     Collection kolabCollection = createCollection( collection );
     CollectionModifyJob *job = new CollectionModifyJob( kolabCollection, this );
+    Q_UNUSED( job );
   } else {
     // Kolab folder we already have in our tree, if the update fails, reload our tree
     Collection kolabCollection = createCollection( collection );
@@ -641,25 +664,43 @@ void KolabProxyResource::imapCollectionRemoved(const Collection &imapCollection)
 Collection KolabProxyResource::createCollection(const Collection& imapCollection)
 {
   Collection c;
-  if ( imapCollection.parentCollection().remoteId() == m_root.remoteId() )
-    c.setParentCollection( m_root );
-  else
+  QStringList contentTypes;
+  if ( imapCollection.parentCollection() == Collection::root() ) {
+    c.setParentCollection( Collection::root() );
+    CachePolicy policy;
+    policy.setInheritFromParent( false );
+    policy.setCacheTimeout( -1 );
+    policy.setLocalParts( QStringList() << QLatin1String( "ALL" ) );
+    c.setCachePolicy( policy );
+  } else {
     c.parentCollection().setRemoteId( QString::number( imapCollection.parentCollection().id() ) );
+  }
+  c.setName( imapCollection.name() );
+  c.setRights( imapCollection.rights() );
+
   EntityDisplayAttribute *imapAttr = imapCollection.attribute<EntityDisplayAttribute>();
   EntityDisplayAttribute *kolabAttr = c.attribute<EntityDisplayAttribute>( Collection::AddIfMissing );
   if ( imapAttr ) {
     if ( imapAttr->iconName() == QLatin1String( "mail-folder-inbox" ) ) {
       kolabAttr->setDisplayName( i18n( "My Data" ) );
       kolabAttr->setIconName( QLatin1String( "view-pim-summary" ) );
+      contentTypes << KolabHandler::allSupportedMimeTypes();
+      c.setRights( Collection::ReadOnly | Collection::CanCreateCollection );
+    } else if ( imapCollection.parentCollection() == Collection::root() ) {
+      c.setName( i18n( "Kolab (%1)", imapAttr->displayName() ) );
+      kolabAttr->setIconName( QLatin1String( "kolab" ) );
     } else {
       kolabAttr->setDisplayName( imapAttr->displayName() );
       kolabAttr->setIconName( imapAttr->iconName() );
     }
+  } else {
+    if ( imapCollection.parentCollection() == Collection::root() ) {
+      c.setName( i18n( "Kolab (%1)", imapCollection.name() ) );
+      kolabAttr->setIconName( QLatin1String( "kolab" ) );
+    }
   }
-  c.setName( imapCollection.name() );
   applyAttributesFromImap( c, imapCollection );
   KolabHandler *handler = m_monitoredCollections.value(imapCollection.id());
-  QStringList contentTypes;
   contentTypes.append( Collection::mimeType() );
   if ( handler ) {
     contentTypes.append( handler->contentMimeTypes() );
@@ -673,7 +714,6 @@ Collection KolabProxyResource::createCollection(const Collection& imapCollection
     }
   }
   c.setContentMimeTypes( contentTypes );
-  c.setRights( imapCollection.rights() );
   c.setRemoteId(QString::number(imapCollection.id()));
   return c;
 }
@@ -685,7 +725,7 @@ bool KolabProxyResource::registerHandlerForCollection(const Akonadi::Collection&
   if ( annotationsAttribute ) {
     QMap<QByteArray, QByteArray> annotations = annotationsAttribute->annotations();
 
-    KolabHandler *handler = KolabHandler::createHandler(annotations["/vendor/kolab/folder-type"]);
+    KolabHandler *handler = KolabHandler::createHandler(annotations["/vendor/kolab/folder-type"], imapCollection );
     if ( handler ) {
       connect(handler, SIGNAL(deleteItemFromImap(const Akonadi::Item&)), this, SLOT(deleteImapItem(const Akonadi::Item&)));
       connect(handler, SIGNAL(addItemToImap(const Akonadi::Item&, Akonadi::Entity::Id)), this, SLOT(addImapItem(const Akonadi::Item&, Akonadi::Entity::Id)));
