@@ -28,8 +28,8 @@
 
 #include <KDebug>
 #include <kde_file.h>
-#include "messagestatus.h"
-using KPIM::MessageStatus;
+#include <akonadi/kmime/messagestatus.h>
+using Akonadi::MessageStatus;
 #include <QFile>
 
 #include <boost/shared_ptr.hpp>
@@ -123,18 +123,12 @@ MessageStatus& KMIndexData::status()
               case KMLegacyMsgStatusUnknown:
                   mStatus.clear();
                   break;
-              case KMLegacyMsgStatusNew:
-                  mStatus.setNew();
-                  break;
               case KMLegacyMsgStatusUnread:
-                  mStatus.setUnread();
+                  mStatus.setRead( false );
                   break;
               case KMLegacyMsgStatusRead:
                   mStatus.setRead();
                   break;
-              case KMLegacyMsgStatusOld:
-                  mStatus.setOld();
-                  break;
               case KMLegacyMsgStatusDeleted:
                   mStatus.setDeleted();
                   break;
