@@ -36,8 +36,8 @@
 #include "filestore/storecompactjob.h"
 
 #include "libmaildir/maildir.h"
-#include "libmbox/mbox.h"
 
+#include <kmbox/mbox.h>
 #include <kmime/kmime_message.h>
 
 #include <akonadi/kmime/messageparts.h>
@@ -55,12 +55,13 @@
 
 using namespace Akonadi;
 using KPIM::Maildir;
+using namespace KMBox;
 
-static bool operator==( const MsgEntryInfo &a, const MsgEntryInfo &b )
+static bool fullEntryCompare( const KMBox::MBoxEntry &left, const KMBox::MBoxEntry &right )
 {
-  return a.offset == b.offset &&
-         a.separatorSize == b.separatorSize &&
-         a.entrySize == b.entrySize;
+  return ( left.messageOffset() == right.messageOffset() &&
+           left.separatorSize() == right.separatorSize() &&
+           left.messageSize() == right.messageSize() );
 }
 
 static bool indexFileForFolder( const QFileInfo &folderDirInfo, QFileInfo &indexFileInfo )
@@ -89,9 +90,10 @@ class MBoxContext
     {
       // in case of reload, check if anything changed, otherwise keep deleted entries
       if ( !mDeletedOffsets.isEmpty() && fileName == mMBox.fileName() ) {
-        const QList<MsgEntryInfo> currentEntryList = mMBox.entryList();
+        const KMBox::MBoxEntry::List currentEntryList = mMBox.entries();
         if ( mMBox.load( fileName ) ) {
-          if ( currentEntryList != mMBox.entryList() ) {
+          const KMBox::MBoxEntry::List newEntryList = mMBox.entries();
+          if ( !std::equal( currentEntryList.begin(), currentEntryList.end(), newEntryList.begin(), fullEntryCompare ) ) {
             mDeletedOffsets.clear();
           }
           return true;
@@ -104,11 +106,11 @@ class MBoxContext
       return mMBox.load( fileName );
     }
 
-    QList<MsgEntryInfo> entryList() const
+    KMBox::MBoxEntry::List entryList() const
     {
-      QList<MsgEntryInfo> result;
-      Q_FOREACH( const MsgEntryInfo &entry, mMBox.entryList() ) {
-        if ( !mDeletedOffsets.contains( entry.offset ) ) {
+      KMBox::MBoxEntry::List result;
+      Q_FOREACH( const KMBox::MBoxEntry &entry, mMBox.entries() ) {
+        if ( !mDeletedOffsets.contains( entry.messageOffset() ) ) {
           result << entry;
         }
       }
@@ -117,23 +119,23 @@ class MBoxContext
 
     QByteArray readRawEntry( quint64 offset )
     {
-      return mMBox.readRawEntry( offset );
+      return mMBox.readRawMessage( KMBox::MBoxEntry( offset ) );
     }
 
     QByteArray readEntryHeaders( quint64 offset )
     {
-      return mMBox.readEntryHeaders( offset );
+      return mMBox.readMessageHeaders( KMBox::MBoxEntry( offset ) );
     }
 
-    qint64 appendEntry( const MessagePtr &entry )
+    qint64 appendEntry( const KMime::Message::Ptr &entry )
     {
-      const qint64 result = mMBox.appendEntry( entry );
-      if ( result >= 0 && mHasIndexData ) {
-        mIndexData.insert( result, KMIndexDataPtr( new KMIndexData ) );
-        Q_ASSERT( mIndexData.value( result )->isEmpty() );
+      const KMBox::MBoxEntry result = mMBox.appendMessage( entry );
+      if ( result.isValid() && mHasIndexData ) {
+        mIndexData.insert( result.messageOffset(), KMIndexDataPtr( new KMIndexData ) );
+        Q_ASSERT( mIndexData.value( result.messageOffset() )->isEmpty() );
       }
 
-      return result;
+      return result.messageOffset();
     }
 
     void deleteEntry( quint64 offset )
@@ -147,8 +149,8 @@ class MBoxContext
         return false;
       }
 
-      Q_FOREACH( const MsgEntryInfo &entry, mMBox.entryList() ) {
-        if ( entry.offset == offset ) {
+      Q_FOREACH( const KMBox::MBoxEntry &entry, mMBox.entries() ) {
+        if ( entry.messageOffset() == offset ) {
           return true;
         }
       }
@@ -161,10 +163,15 @@ class MBoxContext
       return mMBox.save();
     }
 
-    int purge( QList<MsgInfo> &movedEntries )
+    int purge( QList<KMBox::MBoxEntry::Pair> &movedEntries )
     {
       const int deleteCount = mDeletedOffsets.count();
-      const bool result = mMBox.purge( mDeletedOffsets, &movedEntries );
+
+      KMBox::MBoxEntry::List deletedEntries;
+      Q_FOREACH( quint64 offset, mDeletedOffsets )
+        deletedEntries << KMBox::MBoxEntry( offset );
+
+      const bool result = mMBox.purge( deletedEntries, &movedEntries );
 
       if ( mHasIndexData ) {
         // keep copy of original for lookup
@@ -178,14 +185,14 @@ class MBoxContext
         // delete index data for changed entries
         // readded below in an extra loop to handled cases where a new index is equal to an
         // old index of a different entry
-        Q_FOREACH( const MsgInfo &entry, movedEntries ) {
-          mIndexData.remove( entry.first );
+        Q_FOREACH( const KMBox::MBoxEntry::Pair &entry, movedEntries ) {
+          mIndexData.remove( entry.first.messageOffset() );
         }
 
         // readd index data for changed entries at their new position
-        Q_FOREACH( const MsgInfo &entry, movedEntries ) {
-          const KMIndexDataPtr data = indexData.value( entry.first );
-          mIndexData.insert( entry.second, data );
+        Q_FOREACH( const KMBox::MBoxEntry::Pair &entry, movedEntries ) {
+          const KMIndexDataPtr data = indexData.value( entry.first.messageOffset() );
+          mIndexData.insert( entry.second.messageOffset(), data );
         }
       }
 
@@ -278,12 +285,12 @@ void MBoxContext::readIndexData()
 
   mHasIndexData = true;
 
-  const QList<MsgEntryInfo> entries = mMBox.entryList();
-  Q_FOREACH( const MsgEntryInfo &entry, entries ) {
-    const quint64 indexOffset = entry.offset + entry.separatorSize;
+  const KMBox::MBoxEntry::List entries = mMBox.entries();
+  Q_FOREACH( const KMBox::MBoxEntry &entry, entries ) {
+    const quint64 indexOffset = entry.messageOffset() + entry.separatorSize();
     const KMIndexDataPtr data = indexReader.dataByOffset( indexOffset );
     if ( data != 0 ) {
-      mIndexData.insert( entry.offset, data );
+      mIndexData.insert( entry.messageOffset(), data );
     }
   }
 
@@ -562,6 +569,7 @@ MixedMaildirStore::Private::FolderType MixedMaildirStore::Private::folderForColl
       return InvalidFolder;
     }
   }
+  return InvalidFolder;
 }
 
 MBoxPtr MixedMaildirStore::Private::getOrCreateMBoxPtr( const QString &path )
@@ -698,17 +706,17 @@ void MixedMaildirStore::Private::listCollection( FileStore::Job *job, MBoxPtr &m
   QHash<QString, QVariant> uidHash;
   QHash<QString, QVariant> tagListHash;
 
-  const QList<MsgEntryInfo> entryList = mbox->entryList();
-  Q_FOREACH( const MsgEntryInfo &entry, entryList ) {
+  const KMBox::MBoxEntry::List entryList = mbox->entryList();
+  Q_FOREACH( const KMBox::MBoxEntry &entry, entryList ) {
     Item item;
     item.setMimeType( KMime::Message::mimeType() );
-    item.setRemoteId( QString::number( entry.offset ) );
+    item.setRemoteId( QString::number( entry.messageOffset() ) );
     item.setParentCollection( collection );
 
     if ( mbox->hasIndexData() ) {
-      const KMIndexDataPtr indexData = mbox->indexData( entry.offset );
+      const KMIndexDataPtr indexData = mbox->indexData( entry.messageOffset() );
       if ( indexData != 0 && !indexData->isEmpty() ) {
-        item.setFlags( indexData->status().getStatusFlags() );
+        item.setFlags( indexData->status().statusFlags() );
 
         quint64 uid = indexData->uid();
         if ( uid != 0 ) {
@@ -723,9 +731,9 @@ void MixedMaildirStore::Private::listCollection( FileStore::Job *job, MBoxPtr &m
           tagListHash.insert( item.remoteId(), tagList );
         }
       } else if ( indexData == 0 ) {
-        KPIM::MessageStatus status;
+        Akonadi::MessageStatus status;
         status.setDeleted( true ),
-        item.setFlags( status.getStatusFlags() );
+        item.setFlags( status.statusFlags() );
         kDebug() << "no index for item" << item.remoteId() << "in MBox" << mbox->fileName()
                  << "so it has been deleted but not purged. Marking it as"
                  << item.flags();
@@ -767,7 +775,7 @@ void MixedMaildirStore::Private::listCollection( FileStore::Job *job, MaildirPtr
     if ( md->hasIndexData() ) {
       const KMIndexDataPtr indexData = md->indexData( entry );
       if ( indexData != 0 && !indexData->isEmpty() ) {
-        item.setFlags( indexData->status().getStatusFlags() );
+        item.setFlags( indexData->status().statusFlags() );
 
         const quint64 uid = indexData->uid();
         if ( uid != 0 ) {
@@ -819,6 +827,7 @@ bool MixedMaildirStore::Private::fillItem( MBoxPtr &mbox, bool includeBody, Item
     const QByteArray data = mbox->readEntryHeaders( offset );
     messagePtr->setHead( KMime::CRLFtoLF( data ) );
   }
+  messagePtr->parse();
 
   item.setPayload<KMime::Message::Ptr>( messagePtr );
   return true;
@@ -844,6 +853,7 @@ bool MixedMaildirStore::Private::fillItem( const MaildirPtr &md, bool includeBod
 
     messagePtr->setHead( KMime::CRLFtoLF( data ) );
   }
+  messagePtr->parse();
 
   item.setPayload<KMime::Message::Ptr>( messagePtr );
   return true;
@@ -930,11 +940,12 @@ bool MixedMaildirStore::Private::visit( FileStore::CollectionCreateJob *job )
     return false;
   }
 
+  const QString collectionName = job->collection().name().replace( QDir::separator(), QString() );
   Maildir md;
   if ( folderType == MBoxFolder ) {
     const QString subDirPath = Maildir::subDirPathForFolderPath( path );
     const QDir dir( subDirPath );
-    const QFileInfo dirInfo( dir, job->collection().name() );
+    const QFileInfo dirInfo( dir, collectionName );
     if ( dirInfo.exists() && !dirInfo.isDir() ) {
       errorText = i18nc( "@info:status", "Cannot create folder %1 inside folder %2",
                           job->collection().name(), job->targetParent().name() );
@@ -943,7 +954,7 @@ bool MixedMaildirStore::Private::visit( FileStore::CollectionCreateJob *job )
       return false;
     }
 
-    if ( !dir.mkpath( job->collection().name() ) ) {
+    if ( !dir.mkpath( collectionName ) ) {
       errorText = i18nc( "@info:status", "Cannot create folder %1 inside folder %2",
                           job->collection().name(), job->targetParent().name() );
       kError() << errorText << "FolderType=" << folderType << ", mkpath failed";
@@ -964,7 +975,7 @@ bool MixedMaildirStore::Private::visit( FileStore::CollectionCreateJob *job )
     mMaildirs.insert( md.path(), mdPtr );
   } else {
     Maildir parentMd( path, folderType == TopLevelFolder );
-    if ( parentMd.addSubFolder( job->collection().name() ).isEmpty() ) {
+    if ( parentMd.addSubFolder( collectionName ).isEmpty() ) {
       errorText = i18nc( "@info:status", "Cannot create folder %1 inside folder %2",
                           job->collection().name(), job->targetParent().name() );
       kError() << errorText << "FolderType=" << folderType;
@@ -972,13 +983,14 @@ bool MixedMaildirStore::Private::visit( FileStore::CollectionCreateJob *job )
       return false;
     }
 
-    md = Maildir( parentMd.subFolder( job->collection().name() ) );
+    md = Maildir( parentMd.subFolder( collectionName ) );
     const MaildirPtr mdPtr( new MaildirContext( md ) );
     mMaildirs.insert( md.path(), mdPtr );
   }
 
   Collection collection = job->collection();
-  collection.setRemoteId( collection.name() );
+  collection.setRemoteId( collectionName );
+  collection.setName( collectionName );
   collection.setParentCollection( job->targetParent() );
   fillMaildirCollectionDetails( md, collection );
 
@@ -1110,6 +1122,7 @@ static Collection updateMBoxCollectionTree( const Collection &collection, const
 bool MixedMaildirStore::Private::visit( FileStore::CollectionModifyJob *job )
 {
   const Collection collection = job->collection();
+  const QString collectionName = collection.name().replace( QDir::separator(), QString() );
 
   // we also only do renames
   if ( collection.remoteId() == collection.name() ) {
@@ -1134,7 +1147,7 @@ bool MixedMaildirStore::Private::visit( FileStore::CollectionModifyJob *job )
   QDir parentDir( path );
   parentDir.cdUp();
 
-  const QFileInfo targetFileInfo( parentDir, collection.name() );
+  const QFileInfo targetFileInfo( parentDir, collectionName );
   const QFileInfo targetSubDirInfo = Maildir::subDirPathForFolderPath( targetFileInfo.absoluteFilePath() );
 
   if ( targetFileInfo.exists() || ( subDirInfo.exists() && targetSubDirInfo.exists() ) ) {
@@ -1213,7 +1226,8 @@ bool MixedMaildirStore::Private::visit( FileStore::CollectionModifyJob *job )
 
     renamedCollection = q->topLevelCollection();
   } else {
-    renamedCollection.setRemoteId( collection.name() );
+    renamedCollection.setRemoteId( collectionName );
+    renamedCollection.setName( collectionName );
   }
 
   // update collections in MBox contexts so they stay usable for purge
@@ -2094,7 +2108,7 @@ bool MixedMaildirStore::Private::visit( FileStore::StoreCompactJob *job )
     // make the index invalid
     mbox->readIndexData();
 
-    QList<MsgInfo> movedEntries;
+    QList<KMBox::MBoxEntry::Pair> movedEntries;
     const int result = mbox->purge( movedEntries );
     if ( result > 0 ) {
       if ( movedEntries.count() > 0 ) {
@@ -2115,9 +2129,9 @@ bool MixedMaildirStore::Private::visit( FileStore::StoreCompactJob *job )
       }
 
       Item::List items;
-      Q_FOREACH( const MsgInfo &offsetPair, movedEntries ) {
-        const QString oldRemoteId( QString::number( offsetPair.first ) );
-        const QString newRemoteId( QString::number( offsetPair.second ) );
+      Q_FOREACH( const KMBox::MBoxEntry::Pair &offsetPair, movedEntries ) {
+        const QString oldRemoteId( QString::number( offsetPair.first.messageOffset() ) );
+        const QString newRemoteId( QString::number( offsetPair.second.messageOffset() ) );
 
         Item item;
         item.setRemoteId( oldRemoteId );
