@@ -23,12 +23,14 @@
 
 #include <Akonadi/CollectionFetchJob>
 #include <Akonadi/ItemCreateJob>
+#include <akonadi/kmime/messageflags.h>
 #include <akonadi/kmime/specialmailcollectionsrequestjob.h>
 #include <akonadi/kmime/specialmailcollections.h>
+#include <kmime/kmime_util.h>
 #include <Mailtransport/PrecommandJob>
 #include <Mailtransport/Transport>
 
-#include <KIO/PasswordDialog>
+#include <KPasswordDialog>
 #include <KMessageBox>
 #include <kwallet.h>
 
@@ -45,7 +47,8 @@ POP3Resource::POP3Resource( const QString &id )
       mAskAgain( false ),
       mIntervalTimer( new QTimer( this ) ),
       mTestLocalInbox( false ),
-      mWallet( 0 )
+      mWallet( 0 ),
+      mErrorDialogShown( false )
 {
   setNeedsNetwork( true );
   Settings::self()->setResourceId( identifier() );
@@ -81,10 +84,14 @@ void POP3Resource::updateIntervalTimer()
 
 void POP3Resource::intervalCheckTriggered()
 {
-  kDebug() << "Starting interval mail check.";
   Q_ASSERT( mState == Idle );
-  startMailCheck();
-  mIntervalCheckInProgress = true;
+  if ( isOnline() && !mErrorDialogShown ) {
+    kDebug() << "Starting interval mail check.";
+    startMailCheck();
+    mIntervalCheckInProgress = true;
+  } else {
+    mIntervalTimer->start();
+  }
 }
 
 void POP3Resource::aboutToQuit()
@@ -186,21 +193,24 @@ void POP3Resource::walletOpenedForSaving( bool success )
 
 void POP3Resource::showPasswordDialog( const QString &queryText )
 {
-  QString login = Settings::self()->login();
-  bool rememberPassword = Settings::self()->storePassword();
-
   // FIXME: give this a proper parent widget
-  if ( KIO::PasswordDialog::getNameAndPassword(
-       login, mPassword, &rememberPassword, queryText,
-       false, name(), name(), i18n( "Account:" ) ) != KDialog::Accepted )
-    {
+  KPasswordDialog dlg( 0, KPasswordDialog::ShowUsernameLine | KPasswordDialog::ShowKeepPassword );
+  dlg.setUsername( Settings::self()->login() );
+  dlg.setPassword( mPassword );
+  dlg.setKeepPassword( Settings::self()->storePassword() );
+  dlg.setPrompt( queryText );
+  dlg.setCaption( name() );
+  dlg.addCommentLine( i18n( "Account:" ), name() );
+
+  if ( dlg.exec() != KDialog::Accepted ) {
     cancelSync( i18n( "No username and password supplied." ) );
     return;
   } else {
-    Settings::self()->setLogin( login );
+    mPassword = dlg.password();
+    Settings::self()->setLogin( dlg.username() );
     Settings::self()->writeConfig();
     Settings::self()->setStorePassword( false );
-    if ( rememberPassword ) {
+    if ( dlg.keepPassword() ) {
       // setStorePassword( true ) is called only after the password is written into
       // the wallet, as otherwise, the resource thinks the password is in the wallet
       // and loads an empty password from it
@@ -510,6 +520,10 @@ void POP3Resource::loginJobResult( KJob *job )
       mAskAgain = true;
     cancelSync( i18n( "Unable to login to the server %1.", Settings::self()->host() ) +
                 '\n' + job->errorString() );
+
+    // FIXME: "The server refused the supplied username and password." is not correct! The server might
+    //        not even be online, there might be a connection problem etc
+    mErrorDialogShown = true;
     int i = KMessageBox::questionYesNoCancelWId( winIdForDialogs(),
                                   i18n( "The server refused the supplied username and password. "
                                         "Do you want to go to the settings, have another attempt "
@@ -518,6 +532,7 @@ void POP3Resource::loginJobResult( KJob *job )
                                   i18n( "Could Not Authenticate" ),
                                   KGuiItem( i18n( "Settings" ) ),
                                   KGuiItem( i18nc( "Input username/password manually and not store them", "Single Input" ) ) );
+    mErrorDialogShown = false;
     if ( i == KMessageBox::Yes ) {
       configure( winIdForDialogs() );
       return;
@@ -604,6 +619,16 @@ void POP3Resource::messageFinished( int messageId, KMime::Message::Ptr message )
   item.setMimeType( "message/rfc822" );
   item.setPayload<KMime::Message::Ptr>( message );
 
+  // update status flags
+  if ( KMime::isSigned( message.get() ) )
+    item.setFlag( Akonadi::MessageFlags::Signed );
+  if ( KMime::isEncrypted( message.get() ) )
+    item.setFlag( Akonadi::MessageFlags::Encrypted );
+  if ( KMime::isInvitation( message.get() ) )
+    item.setFlag( Akonadi::MessageFlags::HasInvitation );
+  if ( KMime::hasAttachment( message.get() ) )
+    item.setFlag( Akonadi::MessageFlags::HasAttachment );
+
   ItemCreateJob *itemCreateJob = new ItemCreateJob( item, mTargetCollection );
 
   mPendingCreateJobs.insert( itemCreateJob, messageId );
@@ -973,7 +998,7 @@ void POP3Resource::startMailCheck()
 
 void POP3Resource::retrieveCollections()
 {
-  if ( mState == Idle ) {
+  if ( mState == Idle && !mErrorDialogShown ) {
     startMailCheck();
   }
   else {
@@ -982,4 +1007,10 @@ void POP3Resource::retrieveCollections()
   }
 }
 
+void POP3Resource::clearCachedPassword()
+{
+  mPassword.clear();
+}
+
+
 AKONADI_RESOURCE_MAIN( POP3Resource )
